import type { BaseQueryFn, FetchBaseQueryError } from '@reduxjs/toolkit/query';
import { Mutex } from 'async-mutex';

import { getAuthAttributes } from '@procyon/auth/slice';
import { AuthRootState } from '@procyon/auth/types';
import { BaseQueryApi } from '@procyon/types/api';
import { isJSON } from '@procyon/utils';

export type BaseQueryParams = {
	authentication?: { headerName: string; tokenType: string };
	baseUrl: string;
};

export type BaseQueryFetch<Body> = {
	body?: Body;
	headers?: Record<string, string | number | boolean>;
	method?: string;
	params?: Record<string, any>;
	url: string;
};
export type BaseQueryReauthParams = BaseQueryParams & {
	when401: (api: BaseQueryApi) => Promise<void>;
};

const mutex = new Mutex();

export const baseQuery =
	({ baseUrl, authentication }: BaseQueryParams): BaseQueryFn<BaseQueryFetch<any>, unknown, FetchBaseQueryError> =>
	async ({ url, method = 'GET', params, body, headers }, { getState }) => {
		// Build authentication header
		if (authentication && (getState() as AuthRootState).procyon.auth !== undefined) {
			const state = getState() as AuthRootState;
			const { authenticated, accessToken } = getAuthAttributes<unknown>(state);
			if (authenticated) {
				headers = {
					...headers,
					[authentication.headerName]: `${authentication.tokenType} ${accessToken?.toString()}`.trim(),
				};
			}
		}
		// Call endpoint
		try {
			const fetchUrl = new URL(`${baseUrl}${url}`);
			const fetchParams = new URLSearchParams(fetchUrl.search);
			if (params != undefined) {
				for (const [key, value] of Object.entries(params)) {
					fetchParams.append(key, value);
				}
			}
			const response = await fetch(`${fetchUrl.href}?${fetchParams.toString()}`, {
				headers: {
					'Content-Type': 'application/json',
					...headers,
				},
				method: method.toUpperCase(),
				...(body ? { body: JSON.stringify(body) } : {}),
			});
			// Resolve response
			const json = await response.text().then((text: string) => (isJSON(text) ? JSON.parse(text) : text));
			if (!response.ok) throw response;
			return { data: json };
		} catch (err) {
			return { error: { status: err.status, error: err } };
		}
	};

export const baseQueryReauth =
	({
		baseUrl,
		authentication,
		when401,
	}: BaseQueryReauthParams): BaseQueryFn<BaseQueryFetch<any>, unknown, FetchBaseQueryError> =>
	async (args, api, extraOptions) => {
		await mutex.waitForUnlock();
		let result = await baseQuery({ baseUrl, authentication })(args, api, extraOptions);
		if (result.error?.status === 401) {
			if (!mutex.isLocked()) {
				const release = await mutex.acquire();
				try {
					await when401(api);
				} finally {
					release();
				}
			} else {
				await mutex.waitForUnlock();
				result = await baseQuery({ baseUrl, authentication })(args, api, extraOptions);
			}
		}
		return result;
	};
