import React, { useCallback, useEffect, useState } from 'react';
import { omit } from 'ramda';
import { format as formatFn } from 'date-fns';

import { TodayIcon } from '@procyon/components/Icons/Today';
import { ModalId } from '@procyon/components/Modal/context';
import { useModal } from '@procyon/components/Modal/hooks';
import { InputElementType } from '@procyon/types/form';

import { TextField } from '../../TextField';
import { DatePickerModal } from './DatePickerModal';

export const DefaultDateFormat = 'dd.MM.yyyy';

export type DatePickerProps<V = Date> = Omit<
	{
		format?: string;
	} & InputElementType<V>,
	'onFocus' | 'onBlur'
>;

export const DatePicker: React.FC<DatePickerProps> = (props) => {
	const { onChange, label, name, format, ...others } = props;

	const { closeModal, openModal, registerModal } = useModal();

	const [value, setValue] = useState<Date>();

	useEffect(() => {
		if (props.value) {
			setValue(props.value);
		}
	}, [props.value]);

	const handleCloseModal = (id: ModalId) => closeModal(id);

	const handleDateClick = (value: Date): void => {
		setValue(value);
		onChange?.(value, null);
	};

	const modalComponent = useCallback(
		(props) => (
			<DatePickerModal
				format={format ?? DefaultDateFormat}
				name={name}
				value={value}
				closeModal={handleCloseModal}
				onChange={handleDateClick}
				{...props}
			/>
		),
		[format, name, value]
	);

	const handleOpenModal = (): void => {
		registerModal({
			id: name,
			title: label,
			style: { width: 400 },
			Component: modalComponent
		});
		openModal(name);
	};

	return (
		<TextField
			name={name}
			label={label}
			value={value ? formatFn(value, props.format ?? DefaultDateFormat) : undefined}
			{...omit(['value', 'onClick'], others)}
			leftIcon={<TodayIcon onClick={handleOpenModal} className="field-icon" />}
			placeholder={format ?? DefaultDateFormat}
		/>
	);
};
