import React, { FC, useEffect, useMemo } from 'react';
import { useController, useFormContext } from 'react-hook-form';

import { Checkbox } from '@procyon/components/Checkbox';
import { DatePicker } from '@procyon/components/DatePicker';
import { RadioButton, RadioButtonProps } from '@procyon/components/RadioButton';
import { Selectbox, SelectBoxProps } from '@procyon/components/Selectbox';
import { TextField } from '@procyon/components/TextField';
import { TimePicker } from '@procyon/components/TimePicker';
import { FieldTypes } from '@procyon/forms/types';
import { StatusEnum } from '@procyon/types/enums';
import { Option } from '@procyon/types/options';
import { isNilOrEmpty } from '@procyon/utils';

import { ValidationMessage } from './ValidationMessage';

export type FieldProps = {
	name: string;
} & FieldTypes;

export const Field: FC<FieldProps> = (props) => {
	const value = props.component === 'CHECK' ? props.checked : props.value;

	const { control } = useFormContext();

	const {
		field,
		fieldState: { error }
	} = useController({ name: props.name, control });

	useEffect(() => {
		if (!isNilOrEmpty(value)) {
			field.onChange(value);
		}
	}, [value]);

	const handleRadioClick = (code: Option['code'], e: React.MouseEvent<HTMLInputElement>): void => {
		field.onChange(code);
		(props as RadioButtonProps).onClick?.(code, e);
	};

	const handleSelectChange = (option: Option, e: React.MouseEvent<HTMLInputElement>): void => {
		if (isNilOrEmpty(option)) {
			field.onChange(null);
			(props as SelectBoxProps).onChange?.(undefined, e);
		} else {
			field.onChange(option.code);
			(props as SelectBoxProps).onChange?.(option, e);
		}
	};

	const render = useMemo(() => {
		const fieldProps = {
			value: field.value,
			...(error
				? {
						message: <ValidationMessage message={error.message} type={error.type as any} />,
						status: Object.values(StatusEnum).includes(error.type as any)
							? (error.type as StatusEnum)
							: StatusEnum.error
				  }
				: {})
		};
		switch (props.component) {
			case 'TEXT':
				return <TextField {...props} {...fieldProps} onChange={field.onChange} onBlur={field.onBlur} />;
			case 'CHECK':
				return (
					<Checkbox {...props} {...fieldProps} checked={field.value} onChange={field.onChange} onBlur={field.onBlur} />
				);
			case 'RADIO':
				return <RadioButton {...props} {...fieldProps} onClick={handleRadioClick} />;
			case 'SELECT':
				return <Selectbox {...props} {...fieldProps} onChange={handleSelectChange} />;
			case 'DATE':
				return <DatePicker {...props} {...fieldProps} onChange={field.onChange} />;
			case 'TIME':
				return <TimePicker {...props} {...fieldProps} onChange={field.onChange} />;
		}
	}, [props, field, error]);

	return <div className="form-field-container">{render}</div>;
};
